@@ -18,10 +18,11 @@
 #ifndef UWS_LOOP_H
 #define UWS_LOOP_H
 
-/* The loop is lazily created per-thread and run with uWS::run() */
+/* The loop is lazily created per-thread and run with run() */
 
 #include "LoopData.h"
 #include <libusockets.h>
+#include <iostream>
 
 namespace uWS {
 struct Loop {
@@ -56,6 +57,12 @@ private:
         for (auto &p : loopData->postHandlers) {
             p.second((Loop *) loop);
         }
+
+        /* After every event loop iteration, we must not hold the cork buffer */
+        if (loopData->corkedSocket) {
+            std::cerr << "Error: Cork buffer must not be held across event loop iterations!" << std::endl;
+            std::terminate();
+        }
     }
 
     Loop() = delete;
@@ -116,7 +123,7 @@ public:
         getLazyLoop().loop = nullptr;
     }
 
-    void addPostHandler(void *key, fu2::unique_function<void(Loop *)> &&handler) {
+    void addPostHandler(void *key, MoveOnlyFunction<void(Loop *)> &&handler) {
         LoopData *loopData = (LoopData *) us_loop_ext((us_loop_t *) this);
 
         loopData->postHandlers.emplace(key, std::move(handler));
@@ -129,7 +136,7 @@ public:
         loopData->postHandlers.erase(key);
     }
 
-    void addPreHandler(void *key, fu2::unique_function<void(Loop *)> &&handler) {
+    void addPreHandler(void *key, MoveOnlyFunction<void(Loop *)> &&handler) {
         LoopData *loopData = (LoopData *) us_loop_ext((us_loop_t *) this);
 
         loopData->preHandlers.emplace(key, std::move(handler));
@@ -143,7 +150,7 @@ public:
     }
 
     /* Defer this callback on Loop's thread of execution */
-    void defer(fu2::unique_function<void()> &&cb) {
+    void defer(MoveOnlyFunction<void()> &&cb) {
         LoopData *loopData = (LoopData *) us_loop_ext((us_loop_t *) this);
 
         //if (std::thread::get_id() == ) // todo: add fast path for same thread id
